@@ -48,6 +48,5 @@ EXECUTION_MODE_DEFAULT = EXECUTION_MODE_HUB
 EXECUTION_SPOKE_CODEBUILD_PROJECT_NAME = "servicecatalog-puppet-deploy-in-spoke"
 
 
-TASK_OUTPUTS_THAT_SHOULD_NOT_BE_DELETE = [
-    "SharePortfolioTask",
-]
+SHARING_MODE_ACCOUNT = "ACCOUNT"
+SHARING_MODE_AWS_ORGANIZATIONS = "AWS_ORGANIZATIONS"
