@@ -23,15 +23,6 @@ logger = logging.getLogger()
 logger.setLevel(logging.INFO)
 
 
-fh = logging.FileHandler("tasks.log")
-fh.setLevel(logging.INFO)
-formatter = logging.Formatter(
-    "%(levelname)s %(threadName)s [%(filename)s:%(lineno)d] %(message)s"
-)
-fh.setFormatter(formatter)
-logger.addHandler(fh)
-
-
 def run_tasks(
     puppet_account_id,
     current_account_id,
