@@ -12,14 +12,7 @@ import logging
 import math
 
 logger = logging.getLogger("tasks")
-
-fh = logging.FileHandler("tasks.log")
-fh.setLevel(logging.INFO)
-formatter = logging.Formatter(
-    "%(levelname)s %(threadName)s [%(filename)s:%(lineno)d] %(message)s"
-)
-fh.setFormatter(formatter)
-logger.addHandler(fh)
+logger.setLevel(logging.INFO)
 
 
 class PuppetTask(luigi.Task):
